/* This file was automatically generated by seccomp.sh */

#include <stddef.h>
#include <linux/audit.h>
#include <linux/filter.h>
#include <linux/seccomp.h>

#ifndef AUDIT_ARCH_PPC64LE
#define AUDIT_ARCH_PPC64LE (AUDIT_ARCH_PPC64 | __AUDIT_ARCH_LE)
#endif

struct sock_filter filter_passt[] = {
	/* cppcheck-suppress [badBitmaskCheck, unmatchedSuppression] */
	BPF_STMT(BPF_LD | BPF_W | BPF_ABS,
		 (offsetof(struct seccomp_data, arch))),
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, AUDIT_ARCH_X86_64, 0, 80),
	/* cppcheck-suppress [badBitmaskCheck, unmatchedSuppression] */
	BPF_STMT(BPF_LD | BPF_W | BPF_ABS,
		 (offsetof(struct seccomp_data, nr))),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 1, 1, 0),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 2, 1),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 54, 3, 2),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 4, 3),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 5, 4),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 44, 6, 5),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 233, 7, 6),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 10, 7),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 17, 13),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 24, 20),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 31, 27),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 20, 38, 34),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 48, 45, 41),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 77, 52, 48),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 286, 59, 55),
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 64, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 63, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 62, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 61, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 60, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 59, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 58, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 57, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 56, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 55, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 54, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 53, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 52, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 51, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 50, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 49, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 48, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 47, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 46, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 45, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 44, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 43, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 42, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 41, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 40, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 39, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 38, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 37, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 36, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 35, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 34, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 1, 33, 0), /* write */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 3, 32, 0), /* close */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 8, 31, 0), /* lseek */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 16, 30, 0), /* ioctl */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 20, 29, 0), /* writev */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 41, 28, 0), /* socket */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 42, 27, 0), /* connect */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 43, 26, 0), /* accept */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 44, 25, 0), /* sendto */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 45, 24, 0), /* recvfrom */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 46, 23, 0), /* sendmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 47, 22, 0), /* recvmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 48, 21, 0), /* shutdown */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 49, 20, 0), /* bind */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 50, 19, 0), /* listen */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 51, 18, 0), /* getsockname */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 54, 17, 0), /* setsockopt */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 55, 16, 0), /* getsockopt */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 72, 15, 0), /* fcntl */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 74, 14, 0), /* fsync */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 77, 13, 0), /* ftruncate */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 228, 12, 0), /* clock_gettime */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 231, 11, 0), /* exit_group */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 232, 10, 0), /* epoll_wait */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 233, 9, 0), /* epoll_ctl */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 281, 8, 0), /* epoll_pwait */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 283, 7, 0), /* timerfd_create */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 285, 6, 0), /* fallocate */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 286, 5, 0), /* timerfd_settime */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 287, 4, 0), /* timerfd_gettime */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 288, 3, 0), /* accept4 */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 299, 2, 0), /* recvmmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 307, 1, 0), /* sendmmsg */
	BPF_STMT(BPF_RET | BPF_K, SECCOMP_RET_KILL_PROCESS),
	BPF_STMT(BPF_RET | BPF_K, SECCOMP_RET_ALLOW),
};

struct sock_filter filter_pasta[] = {
	/* cppcheck-suppress [badBitmaskCheck, unmatchedSuppression] */
	BPF_STMT(BPF_LD | BPF_W | BPF_ABS,
		 (offsetof(struct seccomp_data, arch))),
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, AUDIT_ARCH_X86_64, 0, 80),
	/* cppcheck-suppress [badBitmaskCheck, unmatchedSuppression] */
	BPF_STMT(BPF_LD | BPF_W | BPF_ABS,
		 (offsetof(struct seccomp_data, nr))),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 43, 1, 0),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 2, 1),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 228, 3, 2),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 4, 3),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 3, 5, 4),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 51, 6, 5),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 283, 7, 6),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 10, 7),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 17, 13),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 24, 20),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 16, 31, 27),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 47, 38, 34),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 60, 45, 41),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 247, 52, 48),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 288, 59, 55),
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 64, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 63, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 62, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 61, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 60, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 59, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 58, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 57, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 56, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 55, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 54, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 53, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 52, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 51, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 50, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 49, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 48, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 47, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 46, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 45, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 44, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 43, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 1, 42, 0), /* write */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 3, 41, 0), /* close */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 8, 40, 0), /* lseek */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 14, 39, 0), /* rt_sigprocmask */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 15, 38, 0), /* rt_sigreturn */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 16, 37, 0), /* ioctl */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 20, 36, 0), /* writev */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 41, 35, 0), /* socket */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 42, 34, 0), /* connect */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 43, 33, 0), /* accept */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 44, 32, 0), /* sendto */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 45, 31, 0), /* recvfrom */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 46, 30, 0), /* sendmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 47, 29, 0), /* recvmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 48, 28, 0), /* shutdown */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 49, 27, 0), /* bind */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 50, 26, 0), /* listen */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 51, 25, 0), /* getsockname */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 54, 24, 0), /* setsockopt */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 55, 23, 0), /* getsockopt */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 56, 22, 0), /* clone */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 60, 21, 0), /* exit */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 72, 20, 0), /* fcntl */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 74, 19, 0), /* fsync */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 77, 18, 0), /* ftruncate */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 228, 17, 0), /* clock_gettime */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 231, 16, 0), /* exit_group */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 232, 15, 0), /* epoll_wait */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 233, 14, 0), /* epoll_ctl */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 247, 13, 0), /* waitid */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 257, 12, 0), /* openat */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 275, 11, 0), /* splice */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 281, 10, 0), /* epoll_pwait */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 283, 9, 0), /* timerfd_create */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 285, 8, 0), /* fallocate */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 286, 7, 0), /* timerfd_settime */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 287, 6, 0), /* timerfd_gettime */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 288, 5, 0), /* accept4 */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 293, 4, 0), /* pipe2 */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 299, 3, 0), /* recvmmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 307, 2, 0), /* sendmmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 308, 1, 0), /* setns */
	BPF_STMT(BPF_RET | BPF_K, SECCOMP_RET_KILL_PROCESS),
	BPF_STMT(BPF_RET | BPF_K, SECCOMP_RET_ALLOW),
};

struct sock_filter filter_vu[] = {
	/* cppcheck-suppress [badBitmaskCheck, unmatchedSuppression] */
	BPF_STMT(BPF_LD | BPF_W | BPF_ABS,
		 (offsetof(struct seccomp_data, arch))),
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, AUDIT_ARCH_X86_64, 0, 80),
	/* cppcheck-suppress [badBitmaskCheck, unmatchedSuppression] */
	BPF_STMT(BPF_LD | BPF_W | BPF_ABS,
		 (offsetof(struct seccomp_data, nr))),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 8, 1, 0),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 2, 1),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 54, 3, 2),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 4, 3),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 5, 4),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 44, 6, 5),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 233, 7, 6),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 10, 7),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 17, 13),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 24, 20),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 31, 27),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 20, 38, 34),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 48, 45, 41),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 77, 52, 48),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 286, 59, 55),
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 64, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 63, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 62, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 61, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 60, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 59, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 58, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 57, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 56, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 55, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 54, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 53, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 52, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 51, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 50, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 49, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 48, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 47, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 46, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 45, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 44, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 43, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 42, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 41, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 40, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 39, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 38, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 37, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 36, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 1, 35, 0), /* write */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 3, 34, 0), /* close */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 8, 33, 0), /* lseek */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 9, 32, 0), /* mmap */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 11, 31, 0), /* munmap */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 16, 30, 0), /* ioctl */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 20, 29, 0), /* writev */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 41, 28, 0), /* socket */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 42, 27, 0), /* connect */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 43, 26, 0), /* accept */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 44, 25, 0), /* sendto */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 45, 24, 0), /* recvfrom */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 46, 23, 0), /* sendmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 47, 22, 0), /* recvmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 48, 21, 0), /* shutdown */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 49, 20, 0), /* bind */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 50, 19, 0), /* listen */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 51, 18, 0), /* getsockname */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 54, 17, 0), /* setsockopt */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 55, 16, 0), /* getsockopt */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 72, 15, 0), /* fcntl */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 74, 14, 0), /* fsync */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 77, 13, 0), /* ftruncate */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 228, 12, 0), /* clock_gettime */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 231, 11, 0), /* exit_group */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 232, 10, 0), /* epoll_wait */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 233, 9, 0), /* epoll_ctl */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 281, 8, 0), /* epoll_pwait */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 283, 7, 0), /* timerfd_create */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 285, 6, 0), /* fallocate */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 286, 5, 0), /* timerfd_settime */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 287, 4, 0), /* timerfd_gettime */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 288, 3, 0), /* accept4 */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 299, 2, 0), /* recvmmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 307, 1, 0), /* sendmmsg */
	BPF_STMT(BPF_RET | BPF_K, SECCOMP_RET_KILL_PROCESS),
	BPF_STMT(BPF_RET | BPF_K, SECCOMP_RET_ALLOW),
};
