/* This file was automatically generated by seccomp.sh */

#include <stddef.h>
#include <linux/audit.h>
#include <linux/filter.h>
#include <linux/seccomp.h>

#ifndef AUDIT_ARCH_PPC64LE
#define AUDIT_ARCH_PPC64LE (AUDIT_ARCH_PPC64 | __AUDIT_ARCH_LE)
#endif

struct sock_filter filter_repair[] = {
	/* cppcheck-suppress [badBitmaskCheck, unmatchedSuppression] */
	BPF_STMT(BPF_LD | BPF_W | BPF_ABS,
		 (offsetof(struct seccomp_data, arch))),
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, AUDIT_ARCH_X86_64, 0, 40),
	/* cppcheck-suppress [badBitmaskCheck, unmatchedSuppression] */
	BPF_STMT(BPF_LD | BPF_W | BPF_ABS,
		 (offsetof(struct seccomp_data, nr))),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 1, 0),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 2, 1),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 45, 3, 2),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 6, 3),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 0, 13, 9),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 5, 20, 16),
	BPF_JUMP(BPF_JMP | BPF_JGE | BPF_K, 231, 27, 23),
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 32, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 31, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 30, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 29, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 28, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 27, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 26, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 25, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 24, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 23, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 22, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 21, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 20, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 19, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 18, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 0, 17, 0), /* read */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 1, 16, 0), /* write */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 3, 15, 0), /* close */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 4, 14, 0), /* stat */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 5, 13, 0), /* fstat */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 41, 12, 0), /* socket */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 42, 11, 0), /* connect */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 44, 10, 0), /* sendto */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 45, 9, 0), /* recvfrom */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 46, 8, 0), /* sendmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 47, 7, 0), /* recvmsg */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 54, 6, 0), /* setsockopt */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 231, 5, 0), /* exit_group */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 254, 4, 0), /* inotify_add_watch */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 262, 3, 0), /* newfstatat */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 294, 2, 0), /* inotify_init1 */
	BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, 332, 1, 0), /* statx */
	BPF_STMT(BPF_RET | BPF_K, SECCOMP_RET_KILL_PROCESS),
	BPF_STMT(BPF_RET | BPF_K, SECCOMP_RET_ALLOW),
};
